package productlogic

import (
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/usecase"
	"context"

	"code.30cm.net/digimon/app-cloudep-product-service/gen_result/pb/product"
	"code.30cm.net/digimon/app-cloudep-product-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListCategoryLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewListCategoryLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListCategoryLogic {
	return &ListCategoryLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// ListCategory 建立 product 分類
func (l *ListCategoryLogic) ListCategory(in *product.ListCategoryReq) (*product.ListCategoryResp, error) {
	q := usecase.CategoryQueryParams{
		PageSize:  in.GetPageSize(),
		PageIndex: in.GetPageIndex(),
	}

	if len(in.GetIds()) > 0 {
		q.ID = in.GetIds()
	}

	categorys, total, err := l.svcCtx.CategoryUseCase.ListCategory(l.ctx, q)
	if err != nil {
		return nil, err
	}

	res := make([]*product.Category, 0, len(categorys))
	for _, category := range categorys {
		res = append(res, &product.Category{
			Id:         category.ID.Hex(),
			Name:       category.Name,
			CreateTime: category.CreatedAt,
			UpdateTime: category.UpdatedAt,
		})
	}

	return &product.ListCategoryResp{
		Total: total,
		Data:  res,
	}, nil
}
