package repository

import (
	"context"
	"time"

	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/entity"
	"code.30cm.net/digimon/app-cloudep-product-service/pkg/domain/repository"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

func (repo *TagsRepository) BindTags(ctx context.Context, data []*entity.TagsBindingTable) error {
	now := time.Now().UTC().UnixNano()
	docs := make([]any, 0, len(data))

	for i := range data {
		if data[i].ID.IsZero() {
			data[i].ID = primitive.NewObjectID()
			data[i].CreatedAt = now
			data[i].UpdatedAt = now
		}
		docs = append(docs, data[i])
	}
	if len(docs) == 0 {
		return nil
	}
	_, err := repo.TageBinding.GetClient().InsertMany(ctx, docs)

	return err
}

func (repo *TagsRepository) UnbindTag(ctx context.Context, tagID, referenceID string) error {
	filter := bson.M{"tag_id": tagID, "reference_id": referenceID}
	_, err := repo.TageBinding.GetClient().DeleteOne(ctx, filter)
	if err != nil {
		return err
	}

	return nil
}

func (repo *TagsRepository) UnbindTagByReferenceID(ctx context.Context, referenceID string) error {
	filter := bson.M{"reference_id": referenceID}
	_, err := repo.TageBinding.GetClient().DeleteMany(ctx, filter)
	if err != nil {
		return err
	}

	return nil
}

func (repo *TagsRepository) GetBindingsByReference(ctx context.Context, referenceID string) ([]*entity.TagsBindingTable, error) {
	var result []*entity.TagsBindingTable
	filter := bson.M{"reference_id": referenceID}
	err := repo.TageBinding.GetClient().Find(ctx, &result, filter)
	if err != nil {
		return nil, err
	}

	return result, nil
}

func (repo *TagsRepository) ListTagBinding(ctx context.Context, params repository.TagBindingQueryParams) ([]*entity.TagsBindingTable, int64, error) {
	// 構建查詢過濾器
	filter := bson.M{}
	if params.TagID != nil {
		filter["tag_id"] = *params.TagID
	}
	if params.ReferenceID != nil {
		filter["reference_id"] = *params.ReferenceID
	}

	// 設置排序選項
	opts := options.Find().SetSkip((params.PageIndex - 1) * params.PageSize).SetLimit(params.PageSize)
	opts.SetSort(bson.D{{Key: "updated_at", Value: -1}})

	// 查詢符合條件的總數
	count, err := repo.TageBinding.GetClient().CountDocuments(ctx, filter)
	if err != nil {
		return nil, 0, err
	}

	// 執行查詢並獲取結果
	var tags []*entity.TagsBindingTable
	err = repo.TageBinding.GetClient().Find(ctx, &tags, filter, opts)
	if err != nil {
		return nil, 0, err
	}

	return tags, count, nil
}

func (repo *TagsRepository) IndexTagsBinding20250317001UP(ctx context.Context) (*mongo.Cursor, error) {
	repo.TageBinding.PopulateMultiIndex(ctx, []string{
		"tag_id",
		"reference_id",
	}, []int32{1, 1}, true)

	return repo.Tags.GetClient().Indexes().List(ctx)
}
